module Jisota

  ##
  # Runs a provision with the given configuration
  #
  # Options allow default depenedencies to be overridden. Example:
  #
  #     Jisota.run(config, logger: Jisota::Output.new(verbose: true))
  #
  def self.run(configuration, options = {})
    provisioner = options.fetch(:provisioner) { Provisioner.new }
    logger      = options.fetch(:logger) { Output.new }
    provisioner.run(configuration, logger)
  end

  ##
  # Shorthand for defining a new configuration
  #
  # Example:
  #
  #     config = Jisota.config do
  #       role :app do
  #         ruby version: "2.1.1"
  #         postgresql
  #       end
  #       server "123.456.789.000", user: "john", roles: :app
  #     end
  def self.config(&block)
    Configuration.new(&block)
  end

  ##
  # Shorthand for defining packages in the global package manager
  # The global package manager is useful when sharing a package between
  # multiple projects or configurations
  #
  # Example:
  #
  #     Jisota.global_config do
  #       package :apt do
  #         param :packages, splat: true, required: true
  #         run { cmd "sudo apt-get install -y #{packages.join(" ")}" }
  #       end
  #     end
  def self.global_config(&block)
    config = config(&block)
    config.packages.each do |package|
      global_packages << package
    end
  end

  ##
  # The global package manager
  #
  # All build-in packages also live here
  # Local packages with the same name will have precedence over these.
  def self.global_packages
    @global_packages ||= Collection.new
  end
end

require_relative 'jisota/dsl_base'
require_relative 'jisota/output'
Dir[File.expand_path("../jisota/*.rb", __FILE__)].each { |file| require file }
Dir[File.expand_path("../jisota/packages/*.rb", __FILE__)].each { |file| require file }
