require 'forwardable'

module Jisota

  ##
  # Stores items with a `:key` attribute
  # In Jisota, this is used for collections of `Package`, `Role` and `Server`
  #
  # Unlike most ruby enumerables, errors are raised when adding duplicate item
  # or getting non-existent item.
  class Collection
    extend Forwardable
    include Enumerable

    class KeyNotFoundError < StandardError; end
    class DuplicateKeyError < StandardError; end

    attr_reader :items

    def_delegators :items, :size, :each, :has_key?

    def initialize(items = {})
      @items = items
    end

    def add(item_with_key)
      key = item_with_key.key
      raise DuplicateKeyError, "Collection already contains an item with key #{key.inspect}" if items.has_key?(key)
      items[key] = item_with_key
    end
    alias_method :<<, :add

    def [](key)
      items.fetch(key) { raise KeyNotFoundError, "The key #{key.inspect} was not found in the collection" }
    end

    def first
      @items.first[1]
    end

    def each(&block)
      @items.values.each(&block)
    end

    def merge(other, &block)
      Collection.new(items.merge(other.items, &block))
    end

  end
end
