require 'forwardable'

module Jisota
  ##
  # Part of the Script duck type
  #
  # Uploads a file on execution
  class FileScript
    attr_accessor :from, :to, :create, :update

    def initialize(from: , to: , create: true, update: true, &block)
      @from = from
      @to = to
      @create = create
      @update = update
      if block_given?
        dsl = DSL.new(self)
        dsl.evaluate(&block)
      end
    end

    def execute(context)
      Executor.new(script: self, context: context).run
    end

    class DSL < DSLBase
      def initialize(target)
        @target = target
      end

      def create(value = nil, &block)
        set_callback(:create=, value, &block)
      end

      def update(value = nil, &block)
        set_callback(:update=, value, &block)
      end

      private

      def set_callback(method, value, &block)
        raise "Can not use both a value and block on create or update" if !value.nil? && block_given?
        if value.nil?
          @target.send(method, ScriptBlock.new(&block))
        else
          @target.send(method, value)
        end
      end
    end

    class Executor
      extend Forwardable
      attr_reader :script, :context
      def_delegators :script, :from, :to, :create, :update
      def_delegators :context, :ssh_session, :logger

      def initialize(script: , context: )
        @script = script
        @context = context
      end

      def tmp_file
        @tmp_file ||= "#{tmp_dir}/#{SecureRandom.hex}"
      end

      def tmp_dir
        "tmp/jisota"
      end

      def run
        logger.system_message("File #{from} -> #{to}") do
          make_temp_dir
          upload
          handle_file_move
        end
      end

      def make_temp_dir
        ssh_session.command("mkdir -p #{tmp_dir}", logger)
      end

      def upload
        logger.system_message("Uploading #{from} -> #{tmp_file}")
        ssh_session.upload(from: from, to: tmp_file)
      end

      def handle_file_move
        result = ssh_session.command("cmp -s #{tmp_file} #{to}")
        case result
        when 0 then same_file
        when 1 then move_with_callback(update, "File exists and update is not allowed. Skipping.")
        when 2 then move_with_callback(create, "File does not exist and create is not allowed. Skipping.")
        end
      end

      def same_file
        logger.system_message("Files are the same. Skipping move.")
        true
      end

      def move_with_callback(callback, callback_not_found_message)
        if callback
          result = ssh_session.command("sudo mkdir -p `dirname #{to}` && sudo mv #{tmp_file} #{to}", logger)
          if result == 0
            if callback.is_a?(ScriptBlock)
              callback_script = callback.evaluate(context)
              callback_script.execute(context)
            else
              true
            end
          end
        else
          logger.system_message(callback_not_found_message)
          true
        end
      end
    end
  end
end
