module Jisota
  ##
  # Part of the Script duck type
  #
  # Executes an entire package, which in turn might execute other packages.
  # The args are parsed with the ParamParser to match args with params.
  #
  # If the package has a `verify_block`, that will be executed first. If the
  # result of verify is success, the `run_block` will not be executed.
  class PackageScript
    attr_accessor :package, :args

    def initialize(package, args = [])
      @package = package
      @args = args
    end

    def execute(context)
      context.param_scope(parse_params) do
        context.logger.system_message("Package #{self}") do
          execute_package(context)
        end
      end
    end

    def to_s
      result = ""
      result << package.name.to_s
      if args.any?
        result << " "
        result << args.map(&:inspect).join(", ")
      end
      result
    end

    private

    def parse_params
      ParamParser.new(package.params, args).parse
    end

    def execute_package(context)
      package.sections.each do |section|
        context.logger.system_message("Section #{section.description}") do
          result = execute_section(section, context)
          return false unless result
        end
      end
      true
    end

    def execute_section(section, context)
      if section.verify_block
        verify_script = section.verify_block.evaluate(context)
        result = verify_script.execute(context)
        if result
          context.logger.system_message("Section #{section.description} already installed")
          true
        else
          execute_run(section, context)
        end
      else
        execute_run(section, context)
      end
    end

    def execute_run(section, context)
      run_script = section.run_block.evaluate(context)
      run_script.execute(context)
    end
  end
end
